"""Embed graphics into HTML Exporter class"""

import base64
import os
import re

from ipython_genutils.ipstruct import Struct
from nbconvert.exporters.html import HTMLExporter

try:
    from urllib.request import urlopen  # py3
except ImportError:
    from urllib2 import urlopen


class EmbedHTMLExporter(HTMLExporter):
    """
    :mod:`nbconvert` Exporter which embeds graphics as base64 into html.

    Convert to HTML and embed graphics (pdf, svg and raster images) in the HTML
    file.

    Example usage::

        jupyter nbconvert --to html_embed mynotebook.ipynb
    """

    def replfunc(self, match):
        """Replace source url or file link with base64 encoded blob."""
        url = match.group(1)
        imgformat = url.split('.')[-1]
        if url.startswith('http'):
            data = urlopen(url).read()
        elif url.startswith('data'):
            img = '<img src="' + url + '"'
            return img
        elif url.startswith('attachment'):
            imgname = url.split(':')[1]
            available_formats = self.attachments[imgname]
            # get the image based on the configured image type priority
            for imgformat in self.config.NbConvertBase.display_data_priority:
                if imgformat in available_formats.keys():
                    b64_data = self.attachments[imgname][imgformat]
                    img = '<img src="data:' + imgformat + \
                          ';base64,' + b64_data + '"'
                    return img
            raise ValueError(
                'Could not find attachment for image "%s" in notebook' %
                imgname)
        else:
            filename = os.path.join(self.path, url)
            with open(filename, 'rb') as f:
                data = f.read()

        self.log.info("embedding url: %s, format: %s" % (url, imgformat))
        b64_data = base64.b64encode(data).decode("utf-8")
        if imgformat == "svg":
            img = '<img src="data:image/svg+xml;base64,' + \
                b64_data + '"'
        elif imgformat == "pdf":
            img = '<img src="data:application/pdf;base64,' + \
                b64_data + '"'
        else:
            img = '<img src="data:image/' + imgformat + \
                ';base64,' + b64_data + '"'
        return img

    def from_notebook_node(self, nb, resources=None, **kw):
        output, resources = super(
            EmbedHTMLExporter, self).from_notebook_node(nb, resources)

        self.path = resources['metadata']['path']
        self.attachments = Struct()
        for cell in nb.cells:
            if 'attachments' in cell.keys():
                self.attachments += cell['attachments']
        regex = re.compile('<img\s+src="([^"]+)"')

        embedded_output = regex.sub(self.replfunc, output)
        return embedded_output, resources



"""Nbconvert exporter to inline css & js for collapsible_headings."""

from __future__ import print_function

from nbconvert.exporters.html import HTMLExporter
from traitlets import Dict


class ExporterInliner(HTMLExporter):

    inliner_resources = Dict(
        {'css': [], 'js': []}, config=True,
        help='css and js scripts to wrap in html <style> or <script> tags')

    def _template_file_default(self):
        return 'inliner'

    def from_notebook_node(self, nb, resources=None, **kw):

        # ensure resources used by template actually exist, add in any from
        # config
        if resources is None:
            resources = {}
        inliner_resources = resources.setdefault('inliner', {})
        for tt in ('css', 'js'):
            existing_items = inliner_resources.setdefault(tt, [])
            existing_items += [
                item for item in self.inliner_resources[tt]
                if item not in existing_items]

        return super(ExporterInliner, self).from_notebook_node(
            nb, resources, **kw)

    @property
    def default_config(self):
        c = super(ExporterInliner, self).default_config
        #  import here to avoid circular import
        from jupyter_contrib_nbextensions.nbconvert_support import (
            templates_directory)
        contrib_templates_dir = templates_directory()

        template_path = c.TemplateExporter.setdefault('template_path', [])
        if contrib_templates_dir not in template_path:
            template_path.append(contrib_templates_dir)

        return c



import os

import jupyter_contrib_nbextensions.nbconvert_support

jcnbe_dir = os.path.dirname(jupyter_contrib_nbextensions.__file__)
pp_mod_name = 'jupyter_contrib_nbextensions.nbconvert_support.pp_highlighter'

c = get_config()  # noqa
c.NbConvertApp.export_format = "html"
c.Exporter.template_path = [
    '.',
    jupyter_contrib_nbextensions.nbconvert_support.templates_directory(),
    os.path.join(jcnbe_dir, 'nbextensions', 'highlighter')
]
c.Exporter.preprocessors = [pp_mod_name + '.HighlighterPreprocessor']
c.NbConvertApp.postprocessor_class = pp_mod_name + '.HighlighterPostProcessor'
# html
c.Exporter.template_file = 'highlighter.tpl'



# -*- coding: utf-8 -*-
"""
This preprocessor removes lines in code cells that have been marked as `folded`
by the codefolding extension
"""

from nbconvert.preprocessors import Preprocessor
from traitlets import Bool, Unicode


class CodeFoldingPreprocessor(Preprocessor):
    """
    :mod:`nbconvert` Preprocessor for the code_folding nbextension.

    Folds codecells as displayed in the notebook. The hidden code below the fold gets removed.

    The preprocessor is installed by default. To enable codefolding with
    NbConvert, you need to set the configuration parameter
    `CodeFoldingPreprocessor.remove_folded_code=True`.
    This can be done either in the `jupyter_nbconvert_config.py` file::

        c.CodeFoldingPreprocessor.remove_folded_code=True = True

    or using a command line parameter when calling NbConvert::

        $ jupyter nbconvert --to html --CodeFoldingPreprocessor.remove_folded_code=True mynotebook.ipynb

    The folding mark can be configured using
            c.CodeFoldingPreprocessor.fold_mark = '<->'

    """  # noqa: E501

    remove_folded_code = Bool(
        False, help="Remove code that was folded").tag(config=True)
    fold_mark = Unicode(u'↔', help="Symbol for folded code").tag(config=True)

    def fold_cell(self, cell, folded):
        """
        Remove folded lines and add a '<->' at the parent line
        """
        self.log.debug("CodeFoldingPreprocessor:: folding at: %s" % folded)
        lines = cell.splitlines(True)
        if not lines:
            # no lines -> return
            return cell
        if folded[0] == 0 and lines[0].startswith(('#', '%')):
            # fold whole cell when first line is a comment or magic
            self.log.debug("fold whole cell")
            return lines[0].rstrip('\n') + self.fold_mark + '\n'

        foldIndent = 0
        fold = False
        fcell = ""
        isSkipLine = False
        skipped = ""

        for i, line in enumerate(lines):
            # fold indent level
            lstrip = line.lstrip(r' \t')  # strip tabs and spaces
            indent = len(line) - len(lstrip)
            # is it a comment or an empty line
            isSkipLine = not lstrip or lstrip == "\n" or lstrip.startswith("#")

            if indent <= foldIndent and not isSkipLine:
                # folding finished, when we reached no skip line on an upper
                # level
                fold = False
                fcell += skipped  # add back all skipped lines
                skipped = ""

            if not fold:
                # append the line
                if i in folded:
                    fold = True
                    foldIndent = indent
                    fcell += line.rstrip('\n') + self.fold_mark + '\n'
                else:
                    fcell += line
            else:
                # dont append the line
                # when folding we track all continuous skip lines
                # to add them back when we leave this scope
                if not isSkipLine:
                    skipped = ""
                else:
                    skipped += line

            self.log.debug("%02i, %02i < %02i, %i,'%s' ", i, indent,
                           foldIndent, fold, line[0:indent + 10].strip("\r\n"))

        return fcell

    def preprocess(self, nb, resources):
        """Skip preprocessor if not enabled"""
        if self.remove_folded_code:
            return super(CodeFoldingPreprocessor, self).preprocess(
                nb, resources)
        else:
            return nb, resources

    def preprocess_cell(self, cell, resources, index):
        """
        Read cell metadata and remove lines marked as `folded`.

        Parameters
        ----------
        cell : NotebookNode cell
            Notebook cell being processed
        resources : dictionary
            Additional resources used in the conversion process.  Allows
            preprocessors to pass variables into the Jinja engine.
        index : int
            Index of the cell being processed (see base.py)
        """
        if hasattr(cell, 'source') and cell.cell_type == 'code':
            if hasattr(cell['metadata'], 'code_folding'):
                folded = cell['metadata']['code_folding']
                if len(folded) > 0:
                    self.log.debug('Removing folded code in cell')
                    cell.source = self.fold_cell(cell.source, folded)
        return cell, resources
